Package: rsolr
Type: Package
Title: R to Solr Interface
Version: 0.0.14
Authors@R: c(
    person("Michael", "Lawrence", role = c("aut", "cre"),
           email = "lawremi@gmail.com"),
    person("Gabe", "Becker", role = "aut"),
    person("Jan", "Vogel", role = "aut")
    )
Description: A comprehensive R API for querying Apache Solr databases.
             A Solr core is represented as a data frame or list that
             supports Solr-side filtering, sorting,
             transformation and aggregation, all through the familiar
             base R API. Queries are processed
             lazily, i.e., a query is only sent to the database when
             the data are required. 
License: Apache License (== 2.0)
VignetteBuilder: knitr
Imports: restfulr (>= 0.0.2), graph, S4Vectors (>= 0.14.3), rjson, XML,
        RCurl
Depends: R (>= 3.4.0), BiocGenerics (>= 0.15.1), methods
Suggests: nycflights13, RUnit, MASS, knitr
Collate: utils.R pminmax.R Context-class.R DocCollection-class.R
        Expression-class.R Facets-class.R FieldInfo-class.R
        FieldType-class.R Promise-class.R SolrExpression-class.R
        SolrQuery-class.R SolrSchema-class.R SolrCore-class.R
        SolrResult-class.R SolrSummary-class.R Solr-class.R
        SolrList-class.R SolrFrame-class.R SolrPromise-class.R
        GroupedSolrFrame-class.R test.R zzz.R
NeedsCompilation: no
Packaged: 2026-05-03 04:04:26 UTC; lawremi
Author: Michael Lawrence [aut, cre],
  Gabe Becker [aut],
  Jan Vogel [aut]
Maintainer: Michael Lawrence <lawremi@gmail.com>
Repository: CRAN
Date/Publication: 2026-05-27 08:00:02 UTC
Built: R 4.5.2; ; 2026-05-27 11:33:18 UTC; unix
